@@ -36,7 +36,10 @@
 
 #ifdef SYS_WINNT
 # include <io.h>
-HANDLE ResolverThreadHandle = NULL;
+static HANDLE ResolverThreadHandle = NULL;
+HANDLE ResolverEventHandle;
+#else
+int resolver_pipe_fd[2];  /* used to let the resolver process alert the parent process */
 #endif /* SYS_WINNT */
 
 /*
@@ -128,6 +131,7 @@ static	struct keyword mod_keywords[] = {
 	{ "prefer",		CONF_MOD_PREFER },
 	{ "ttl",		CONF_MOD_TTL },     /* NTP peers */
 	{ "version",		CONF_MOD_VERSION },
+	{ "dynamic",		CONF_MOD_DYNAMIC },
 	{ "",			CONFIG_UNKNOWN }
 };
 
@@ -357,7 +361,6 @@ static char res_file[MAX_PATH];
  */
 
 short default_ai_family = AF_UNSPEC;	/* Default either IPv4 or IPv6 */
-char const *progname;
 char	*sys_phone[MAXPHONE] = {NULL}; /* ACTS phone numbers */
 char	*keysdir = NTP_KEYSDIR;	/* crypto keys directory */
 #if defined(HAVE_SCHED_SETSCHEDULER)
@@ -406,7 +409,7 @@ static	int matchkey P((char *, struct keyword *, int));
 enum gnn_type {
 	t_UNK,		/* Unknown */
 	t_REF,		/* Refclock */
-	t_MSK,		/* Network Mask */
+	t_MSK		/* Network Mask */
 	};
 static	int getnetnum P((const char *, struct sockaddr_storage *, int,
 			 enum gnn_type));
@@ -525,7 +528,7 @@ getconfig(
 	 * Initialize, initialize
 	 */
 	errflg = 0;
-	/* HMS: don't initialize debug to 0 here! */
+	
 #ifndef SYS_WINNT
 	config_file = CONFIG_FILE;
 #else
@@ -544,7 +547,6 @@ getconfig(
 	alt_config_file = alt_config_file_storage;
 
 #endif /* SYS_WINNT */
-	progname = argv[0];
 	res_fp = NULL;
 	ntp_syslogmask = NLOG_SYNCMASK; /* set more via logconfig */
 
@@ -810,6 +812,13 @@ getconfig(
 				case CONF_MOD_IBURST:
 				    peerflags |= FLAG_IBURST;
 				    break;
+
+				case CONF_MOD_DYNAMIC:
+				    msyslog(LOG_WARNING, 
+				        "Warning: the \"dynamic\" keyword has been obsoleted"
+				        " and will be removed in the next release\n");
+				    break;
+
 #ifdef OPENSSL
 				case CONF_MOD_SKEY:
 				    peerflags |= FLAG_SKEY |
@@ -2202,6 +2211,19 @@ getnetnum(
 	struct addrinfo *ptr;
 	int retval;
 
+#if 0
+	printf("getnetnum: <%s> is a %s (%d)\n",
+		num,
+		(a_type == t_UNK)
+		? "t_UNK"
+		: (a_type == t_REF)
+		  ? "t_REF"
+		  : (a_type == t_MSK)
+		    ? "t_MSK"
+		    : "???",
+		a_type);
+#endif
+
 	/* Get host address. Looking for UDP datagram connection */
  	memset(&hints, 0, sizeof (hints));
  	if (addr->ss_family == AF_INET || addr->ss_family == AF_INET6)
@@ -2215,9 +2237,14 @@ getnetnum(
 		hints.ai_family = AF_INET;
 
 	hints.ai_socktype = SOCK_DGRAM;
+
+	if (a_type != t_UNK) {
+		hints.ai_flags = AI_NUMERICHOST;
+	}
+
 #ifdef DEBUG
-		if (debug > 3)
-			printf("getaddrinfo %s\n", num);
+	if (debug > 3)
+		printf("getnetnum: calling getaddrinfo(%s,...)\n", num);
 #endif
 	retval = getaddrinfo(num, "ntp", &hints, &ptr);
 	if (retval != 0 ||
@@ -2248,8 +2275,16 @@ getnetnum(
 	memcpy(addr, ptr->ai_addr, ptr->ai_addrlen);
 #ifdef DEBUG
 	if (debug > 1)
-		printf("getnetnum given %s, got %s \n",
-		   num, stoa(addr));
+		printf("getnetnum given %s, got %s (%s/%d)\n",
+		   num, stoa(addr),
+			(a_type == t_UNK)
+			? "t_UNK"
+			: (a_type == t_REF)
+			  ? "t_REF"
+			  : (a_type == t_MSK)
+			    ? "t_MSK"
+			    : "???",
+			a_type);
 #endif
         freeaddrinfo(ptr);
 	return 1;
@@ -2402,7 +2437,22 @@ do_resolve_internal(void)
 	(void) signal_no_reset(SIGCHLD, catchchild);
 
 #ifndef SYS_VXWORKS
+	/* the parent process will write to the pipe
+	 * in order to wake up to child process
+	 * which may be waiting in a select() call
+	 * on the read fd */
+	if (pipe(resolver_pipe_fd) < 0) {
+		msyslog(LOG_ERR,
+			"unable to open resolver pipe");
+		exit(1);
+	}
+
 	i = fork();
+	/* Shouldn't the code below be re-ordered?
+	 * I.e. first check if the fork() returned an error, then
+	 * check whether we're parent or child.
+	 *     Martin Burnicki
+	 */
 	if (i == 0) {
 		/*
 		 * this used to close everything
@@ -2437,6 +2487,9 @@ do_resolve_internal(void)
 		 * THUS:
 		 */
 
+		/* This is the child process who will read the pipe,
+		 * so we close the write fd */
+		close(resolver_pipe_fd[1]);
 		closelog();
 		kill_asyncio(0);
 
@@ -2485,6 +2538,11 @@ do_resolve_internal(void)
 		(void) signal_no_reset(SIGCHLD, SIG_DFL);
 		abort_resolve();
 	}
+	else {
+		/* This is the parent process who will write to the pipe,
+		 * so we close the read fd */
+		close(resolver_pipe_fd[0]);
+	}
 #else /* SYS_WINNT */
 	{
 		/* NT's equivalent of fork() is _spawn(), but the start point
@@ -2493,6 +2551,11 @@ do_resolve_internal(void)
 		 */
 		DWORD dwThreadId;
 		fflush(stdout);
+		ResolverEventHandle = CreateEvent(NULL, FALSE, FALSE, NULL);
+		if (ResolverEventHandle == NULL) {
+			msyslog(LOG_ERR, "Unable to create resolver event object, can't start ntp_intres");
+			abort_resolve();
+		}
 		ResolverThreadHandle = CreateThread(
 			NULL,				 /* no security attributes	*/
 			0,				 /* use default stack size	*/
@@ -2502,6 +2565,8 @@ do_resolve_internal(void)
 			&dwThreadId);			 /* returns the thread identifier */
 		if (ResolverThreadHandle == NULL) {
 			msyslog(LOG_ERR, "CreateThread() failed, can't start ntp_intres");
+			CloseHandle(ResolverEventHandle);
+			ResolverEventHandle = NULL;
 			abort_resolve();
 		}
 	}
