@@ -368,7 +368,7 @@ acts_message(
 	up = (struct actsunit *)pp->unitptr;
 #ifdef DEBUG
 	ioctl(pp->io.fd, TIOCMGET, (char *)&modem);
-	sprintf(tbuf, "acts: %04x (%d %d) %d %s", modem, up->state,
+	sprintf(tbuf, "acts: %04x (%d %d) %lu %s", modem, up->state,
 	    up->timer, strlen(pp->a_lastcode), pp->a_lastcode);
 	if (debug)
 		printf("%s\n", tbuf);
