@@ -7,32 +7,27 @@
 #include <config.h>
 #endif
 
-#if defined(SYS_WINNT)
-#undef close
-#define close closesocket
-#endif
-
 #if defined(REFCLOCK) && defined(CLOCK_NMEA)
 
+#include <stdio.h>
+#include <ctype.h>
+
 #include "ntpd.h"
 #include "ntp_io.h"
 #include "ntp_unixtime.h"
 #include "ntp_refclock.h"
 #include "ntp_stdlib.h"
 
-#include <stdio.h>
-#include <ctype.h>
-
 #ifdef HAVE_PPSAPI
-# ifdef HAVE_TIMEPPS_H
-#  include <timepps.h>
-# else
-#  ifdef HAVE_SYS_TIMEPPS_H
-#   include <sys/timepps.h>
-#  endif
-# endif
+# include "ppsapi_timepps.h"
 #endif /* HAVE_PPSAPI */
 
+#ifdef SYS_WINNT
+extern int async_write(int, const void *, unsigned int);
+#undef write
+#define write(fd, data, octets)	async_write(fd, data, octets)
+#endif
+
 /*
  * This driver supports the NMEA GPS Receiver with
  *
@@ -153,8 +148,53 @@ nmea_start(
 	(void)sprintf(device, DEVICE, unit);
 
 	fd = refclock_open(device, SPEED232, LDISC_CLK);
-	if (fd < 0)
-	    return (0);
+	if (fd <= 0) {
+#ifdef HAVE_READLINK
+          /* nmead support added by Jon Miner (cp_n18@yahoo.com)
+           *
+           * See http://home.hiwaay.net/~taylorc/gps/nmea-server/
+           * for information about nmead
+           *
+           * To use this, you need to create a link from /dev/gpsX to
+           * the server:port where nmead is running.  Something like this:
+           *
+           * ln -s server:port /dev/gps1
+           */
+          char buffer[80];
+          char *nmea_host;
+          int   nmea_port;
+          int   len;
+          struct hostent *he;
+          struct protoent *p;
+          struct sockaddr_in so_addr;
+
+          if ((len = readlink(device,buffer,sizeof(buffer))) == -1)
+            return(0);
+          buffer[len] = 0;
+
+          if ((nmea_host = strtok(buffer,":")) == NULL)
+            return(0);
+         
+          nmea_port = atoi(strtok(NULL,":"));
+
+          if ((he = gethostbyname(nmea_host)) == NULL)
+            return(0);
+          if ((p = getprotobyname("ip")) == NULL)
+            return(0);
+          so_addr.sin_family = AF_INET;
+          so_addr.sin_port = htons(nmea_port);
+          so_addr.sin_addr = *((struct in_addr *) he->h_addr);
+
+          if ((fd = socket(PF_INET,SOCK_STREAM,p->p_proto)) == -1)
+            return(0);
+          if (connect(fd,(struct sockaddr *)&so_addr,SOCKLEN(&so_addr)) == -1) {
+            close(fd);
+            return (0);
+          }
+#else
+            return (0);
+#endif
+        }
 
 	/*
 	 * Allocate and initialize unit structure
