@@ -158,11 +158,6 @@ int simple_query = 0;
  */
 int unpriv_port = 0;
 
-/*
- * Time to spend measuring drift rate
- */
-int rate = 0;
-
 /*
  * Program name.
  */
@@ -278,10 +273,6 @@ void clear_globals()
    */
   unpriv_port = 0;
 
-  /*
-   * Time to spend measuring drift rate
-   */
-  rate = 0;
   /*
    * Systemwide parameters and flags
    */
@@ -376,7 +367,7 @@ ntpdatemain (
 	/*
 	 * Decode argument list
 	 */
-	while ((c = ntp_getopt(argc, argv, "46a:bBde:k:o:p:qr:st:uv")) != EOF)
+	while ((c = ntp_getopt(argc, argv, "46a:bBde:k:o:p:qst:uv")) != EOF)
 		switch (c)
 		{
 		case '4':
@@ -432,17 +423,6 @@ ntpdatemain (
 		case 'q':
 			simple_query = 1;
 			break;
-		case 'r':
-			c = atoi(ntp_optarg);
-			if (c <= 0 || c > (60 * 60)) {
-				(void) fprintf(stderr,
-					   "%s: rate (%d) is invalid: 0 - %d\n",
-					   progname, c, (60 * 60));
-				errflg++;
-			} else {
-				rate = c;
-			}
-			break;
 		case 's':
 			syslogit = 1;
 			break;
@@ -474,7 +454,7 @@ ntpdatemain (
 	
 	if (errflg) {
 		(void) fprintf(stderr,
-		    "usage: %s [-46bBdqsuv] [-a key#] [-e delay] [-k file] [-p samples] [-o version#] [-r rate] [-t timeo] server ...\n",
+		    "usage: %s [-46bBdqsuv] [-a key#] [-e delay] [-k file] [-p samples] [-o version#] [-t timeo] server ...\n",
 		    progname);
 		exit(2);
 	}
@@ -1482,7 +1462,7 @@ findserver(
 		server = (struct server *)emalloc(sizeof(struct server));
 		memset((char *)server, 0, sizeof(struct server));
 
-		memcpy(&server->srcadr, &addr, sizeof(struct sockaddr_storage));
+		memcpy(&server->srcadr, addr, sizeof(struct sockaddr_storage));
 
 		server->event_time = ++sys_numservers;
 
