@@ -21,11 +21,13 @@
 #include "isc/net.h"
 #include "isc/result.h"
 
+#include "ntpq-opts.h"
+
 #ifdef SYS_WINNT
-#include <Mswsock.h>
+# include <Mswsock.h>
 # include <io.h>
 #else
-#define closesocket close
+# define closesocket close
 #endif /* SYS_WINNT */
 
 #if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDIT)
@@ -34,9 +36,14 @@
 #endif /* HAVE_LIBREADLINE || HAVE_LIBEDIT */
 
 #ifdef SYS_VXWORKS
-/* vxWorks needs mode flag -casey*/
-#define open(name, flags)   open(name, flags, 0777)
-#define SERVER_PORT_NUM     123
+				/* vxWorks needs mode flag -casey*/
+# define open(name, flags)   open(name, flags, 0777)
+# define SERVER_PORT_NUM     123
+#endif
+
+/* we use COMMAND as an autogen keyword */
+#ifdef COMMAND
+# undef COMMAND
 #endif
 
 /*
@@ -466,10 +473,8 @@ CALL(ntpq,"ntpq",ntpqmain);
 void clear_globals(void)
 {
     extern int ntp_optind;
-    extern char *ntp_optarg;
     showhostnames = 0;				/* don'tshow host names by default */
     ntp_optind = 0;
-    ntp_optarg = 0;
     server_entry = NULL;            /* server entry for ntp */
     havehost = 0;				/* set to 1 when host open */
     numassoc = 0;		/* number of cached associations */
@@ -498,10 +503,7 @@ ntpqmain(
 	char *argv[]
 	)
 {
-	int c;
-	int errflg = 0;
 	extern int ntp_optind;
-	extern char *ntp_optarg;
 
 #ifdef SYS_VXWORKS
 	clear_globals();
@@ -525,7 +527,49 @@ ntpqmain(
 	}
 
 	progname = argv[0];
-	ai_fam_templ = ai_fam_default;
+
+	{
+		int optct = optionProcess(&ntpqOptions, argc, argv);
+		argc -= optct;
+		argv += optct;
+	}
+
+	switch (WHICH_IDX_IPV4) {
+	    case INDEX_OPT_IPV4:
+		ai_fam_templ = AF_INET;
+		break;
+	    case INDEX_OPT_IPV6:
+		ai_fam_templ = AF_INET6;
+		break;
+	    default:
+		ai_fam_templ = ai_fam_default;
+		break;
+	}
+
+	if (HAVE_OPT(COMMAND)) {
+		int		cmdct = STACKCT_OPT( COMMAND );
+		const char**	cmds  = STACKLST_OPT( COMMAND );
+
+		while (cmdct-- > 0) {
+			ADDCMD(*cmds++);
+		}
+	}
+
+	debug = DESC(DEBUG_LEVEL).optOccCt;
+
+	if (HAVE_OPT(INTERACTIVE)) {
+		interactive = 1;
+	}
+
+	if (HAVE_OPT(NUMERIC)) {
+		showhostnames = 0;
+	}
+
+	if (HAVE_OPT(PEERS)) {
+		ADDCMD("peers");
+	}
+
+#if 0
 	while ((c = ntp_getopt(argc, argv, "46c:dinp")) != EOF)
 	    switch (c) {
 		case '4':
@@ -559,6 +603,7 @@ ntpqmain(
 			       progname);
 		exit(2);
 	}
+#endif
 	if (ntp_optind == argc) {
 		ADDHOST(DEFHOST);
 	} else {
@@ -3140,9 +3185,9 @@ cookedprint(
 				if (!decodeuint(value, &uval))
 				    output_raw = '?';
 				else {
-					char b[10];
+					char b[12];
 
-					(void) sprintf(b, "%03lo", uval);
+					(void) snprintf(b, sizeof b, "%03lo", uval);
 					output(fp, name, b);
 				}
 				break;
