@@ -99,6 +99,8 @@
 #include "ntp_random.h"
 #include "l_stdlib.h"
 
+#include "ntp-keygen-opts.h"
+
 #ifdef SYS_WINNT
 extern	int	ntp_getopt	P((int, char **, const char *));
 #define getopt ntp_getopt
@@ -216,8 +218,8 @@ main(
 	char	**argv
 	)
 {
-	int	errflg = 0;
 	struct timeval tv;	/* initialization vector */
+	int	md5key = 0;	/* generate MD5 keys */
 #ifdef OPENSSL
 	X509	*cert = NULL;	/* X509 certificate */
 	EVP_PKEY *pkey_host = NULL; /* host key */
@@ -225,9 +227,6 @@ main(
 	EVP_PKEY *pkey_iff = NULL; /* IFF parameters */
 	EVP_PKEY *pkey_gq = NULL; /* GQ parameters */
 	EVP_PKEY *pkey_mv = NULL; /* MV parameters */
-#endif
-	int	md5key = 0;	/* generate MD5 keys */
-#ifdef OPENSSL
 	int	hostkey = 0;	/* generate RSA keys */
 	int	iffkey = 0;	/* generate IFF parameters */
 	int	gqpar = 0;	/* generate GQ parameters */
@@ -243,10 +242,10 @@ main(
 	char	*grpkey = NULL;	/* identity extension */
 	int	nid;		/* X509 digest/signature scheme */
 	FILE	*fstr = NULL;	/* file handle */
-	int	iffsw = 0;	/* IFF key switch */
+	u_int	temp;
+#define iffsw   HAVE_OPT(ID_KEY)
 #endif /* OPENSSL */
 	char	hostbuf[MAXHOSTNAME + 1];
-	u_int	temp;
 
 #ifdef SYS_WINNT
 	/* Initialize before OpenSSL checks */
@@ -288,229 +287,73 @@ main(
 #endif
 	epoch = tv.tv_sec;
 	rval = 0;
-	while ((temp = getopt(argc, argv,
-#ifdef OPENSSL
-	    "c:deGgHIi:Mm:nPp:q:S:s:TV:v:"
-#else
-	    "dM"
-#endif
-	    )) != -1) {
-		switch(temp) {
 
-#ifdef OPENSSL
-		/*
-		 * -c select public certificate type
-		 */
-		case 'c':
-			scheme = optarg;
-			continue;
-#endif
-
-		/*
-		 * -d debug
-		 */
-		case 'd':
-			debug++;
-			continue;
-
-#ifdef OPENSSL
-		/*
-		 * -e write identity keys
-		 */
-		case 'e':
-			iffsw++;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -G generate GQ parameters and keys
-		 */
-		case 'G':
-			gqpar++;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -g update GQ keys
-		 */
-		case 'g':
-			gqkey++;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -H generate host key (RSA)
-		 */
-		case 'H':
-			hostkey++;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -I generate IFF parameters
-		 */
-		case 'I':
-			iffkey++;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -i set issuer name
-		 */
-		case 'i':
-			trustname = optarg;
-			continue;
-#endif
-
-		/*
-		 * -M generate MD5 keys
-		 */
-		case 'M':
-			md5key++;
-			continue;
-
-#ifdef OPENSSL
-		/*
-		 * -m select modulus (256-2048)
-		 */
-		case 'm':
-			if (sscanf(optarg, "%d", &modulus) != 1) {
-				fprintf(stderr,
-				    "invalid option -m %s\n", optarg);	
-				++errflg;
-			}
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -P generate PC private certificate
-		 */
-		case 'P':
-			exten = EXT_KEY_PRIVATE;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -p output private key password
-		 */
-		case 'p':
-			passwd2 = optarg;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -q input private key password
-		 */
-		case 'q':
-			passwd1 = optarg;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -S generate sign key (RSA or DSA)
-		 */
-		case 'S':
-			sign = optarg;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -s set subject name
-		 */
-		case 's':
-			hostname = optarg;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -T trusted certificate (TC scheme)
-		 */
-		case 'T':
-			exten = EXT_KEY_TRUST;
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -V <keys> generate MV parameters
-		 */
-		case 'V':
-			mvpar++;
-			if (sscanf(optarg, "%d", &nkeys) != 1) {
-				fprintf(stderr,
-				    "invalid option -V %s\n", optarg);
-				++errflg;
-			}
-			continue;
-#endif
-
-#ifdef OPENSSL
-		/*
-		 * -v <key> update MV keys
-		 */
-		case 'v':
-			mvkey++;
-			if (sscanf(optarg, "%d", &nkeys) != 1) {
-				fprintf(stderr,
-				    "invalid option -v %s\n", optarg);
-				++errflg;
-			}
-			continue;
-#endif
-
-		/*
-		 * None of the above.
-		 */
-		default:
-			++errflg;
-			continue;
-		}
+	{
+		int optct = optionProcess(&ntp_keygenOptions, argc, argv);
+		argc -= optct;
+		argv += optct;
 	}
 
-	if (errflg) {
-		printf("Usage:  ntp-keygen [options]\n");
-		printf("where options are:\n");
 #ifdef OPENSSL
-		printf("   -c cert_scheme\n");
+	if (HAVE_OPT( CERTIFICATE ))
+	    scheme = OPT_ARG( CERTIFICATE );
 #endif
-		printf("   -d			increase debug level\n");
+
+	debug = DESC(DEBUG_LEVEL).optOccCt;
+
 #ifdef OPENSSL
-		printf("   -e			Write identity keys\n");
-		printf("   -G			Generate GQ parameters and keys\n");
-		printf("   -g			Update GQ keys\n");
-		printf("   -H			Generate RSA Host key\n");
-		printf("   -I			Generate IFF parameters\n");
-		printf("   -i issuer_name\n");
+	if (HAVE_OPT( GQ_PARAMS ))
+	    gqpar++;
+
+	if (HAVE_OPT( GQ_KEYS ))
+	    gqkey++;
+
+	if (HAVE_OPT( HOST_KEY ))
+	    hostkey++;
+
+	if (HAVE_OPT( IFFKEY ))
+	    iffkey++;
+
+	if (HAVE_OPT( ISSUER_NAME ))
+	    trustname = OPT_ARG( ISSUER_NAME );
 #endif
-		printf("   -M			Generate MD5 keys\n");
+
+	if (HAVE_OPT( MD5KEY ))
+	    md5key++;
+
 #ifdef OPENSSL
-		printf("   -m modulus		256 - 2048\n");
-		printf("   -P			generate PC private certificate\n");
-		printf("   -p output_pass	output private password\n");
-		printf("   -q input_pass	input private password\n");
-		printf("   -S sign-key		generate sign key (RSA or DSA)\n");
-		printf("   -s set-subj-name\n");
-		printf("   -T			Trusted certificate (TC scheme)\n");
-		printf("   -V #keys		generate MV parameters\n");
-		printf("   -v #keys		update MV parameters\n");
-		printf("\n");
-		printf("If there is no new host key, look for an existing one.\n");
-		printf("If one is not found, create it.\n");
-#endif
-		exit(2);
+	if (HAVE_OPT( MODULUS ))
+	    modulus = OPT_VALUE_MODULUS;
+
+	if (HAVE_OPT( PVT_CERT ))
+	    exten = EXT_KEY_PRIVATE;
+
+	if (HAVE_OPT( PVT_PASSWD ))
+	    passwd2 = OPT_ARG( PVT_PASSWD );
+
+	if (HAVE_OPT( GET_PVT_PASSWD ))
+	    passwd1 = OPT_ARG( GET_PVT_PASSWD );
+
+	if (HAVE_OPT( SIGN_KEY ))
+	    sign = OPT_ARG( SIGN_KEY );
+
+	if (HAVE_OPT( SUBJECT_NAME ))
+	    hostname = OPT_ARG( SUBJECT_NAME );
+
+	if (HAVE_OPT( TRUSTED_CERT ))
+	    exten = EXT_KEY_TRUST;
+
+	if (HAVE_OPT( MV_PARAMS )) {
+		mvpar++;
+		nkeys = OPT_VALUE_MV_PARAMS;
 	}
 
+	if (HAVE_OPT( MV_KEYS )) {
+		mvkey++;
+		nkeys = OPT_VALUE_MV_KEYS;
+	}
+#endif
+
 	if (passwd1 != NULL && passwd2 == NULL)
 		passwd2 = passwd1;
 #ifdef OPENSSL
@@ -557,7 +400,7 @@ main(
 	 * If there is no new host key, look for an existing one. If not
 	 * found, create it.
 	 */
-	while (pkey_host == NULL && rval == 0 && !iffsw) {
+	while (pkey_host == NULL && rval == 0 && !HAVE_OPT(ID_KEY)) {
 		sprintf(filename, "ntpkey_host_%s", hostname);
 		if ((fstr = fopen(filename, "r")) != NULL) {
 			pkey_host = PEM_read_PrivateKey(fstr, NULL,
@@ -587,7 +430,7 @@ main(
 	 * found, use the host key instead.
 	 */
 	pkey = pkey_sign;
-	while (pkey_sign == NULL && rval == 0 && !iffsw) {
+	while (pkey_sign == NULL && rval == 0 && !HAVE_OPT(ID_KEY)) {
 		sprintf(filename, "ntpkey_sign_%s", hostname);
 		if ((fstr = fopen(filename, "r")) != NULL) {
 			pkey_sign = PEM_read_PrivateKey(fstr, NULL,
@@ -637,7 +480,7 @@ main(
 	/*
 	 * If there is no new GQ file, look for an existing one.
 	 */
-	if (pkey_gq == NULL && rval == 0 && !iffsw) {
+	if (pkey_gq == NULL && rval == 0 && !HAVE_OPT(ID_KEY)) {
 		sprintf(filename, "ntpkey_gq_%s", hostname);
 		if ((fstr = fopen(filename, "r")) != NULL) {
 			pkey_gq = PEM_read_PrivateKey(fstr, NULL, NULL,
@@ -669,7 +512,7 @@ main(
 	/*
 	 * Generate a X509v3 certificate.
 	 */
-	while (scheme == NULL && rval == 0 && !iffsw) {
+	while (scheme == NULL && rval == 0 && !HAVE_OPT(ID_KEY)) {
 		sprintf(filename, "ntpkey_cert_%s", hostname);
 		if ((fstr = fopen(filename, "r")) != NULL) {
 			cert = PEM_read_X509(fstr, NULL, NULL, NULL);
@@ -692,7 +535,7 @@ main(
 		}
 		scheme = "RSA-MD5";
 	}
-	if (pkey != NULL && rval == 0 && !iffsw) {
+	if (pkey != NULL && rval == 0 && !HAVE_OPT(ID_KEY)) {
 		ectx = EVP_get_digestbyname(scheme);
 		if (ectx == NULL) {
 			fprintf(stderr,
@@ -708,7 +551,7 @@ main(
 	 * Write the IFF client parameters and keys as a DSA private key
 	 * encoded in PEM. Note the private key is obscured.
 	 */
-	if (pkey_iff != NULL && rval == 0 && iffsw) {
+	if (pkey_iff != NULL && rval == 0 && HAVE_OPT(ID_KEY)) {
 		DSA	*dsa;
 		char	*sptr;
 		char	*tld;
